<?php

/*
 * NumitPlugin.class.php - example core plugin
 *
 * Copyright (C) 2006-2010 - Till Glggler <tgloeggl@uos.de>
 * Copyright (C) 2008 - Andr Klaen <aklassen@uos.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */


/**
 * An Administration Plugin which merges two users.
 *
 * @package     virtUOS
 * @subpackage  administration
 *
 * @author    tgloeggl, aklassen
 * @copyright (c) Authors
 * @version   $Id$
 */


require_once ('lib/messaging.inc.php');

class NumitPlugin extends StudIPPlugin implements AdministrationPlugin {

    function __construct()
    {
        if ($GLOBALS['perm']->have_perm('admin')) {
            Navigation::addItem('/admin/tools/numit', new Navigation($this->getPluginName(), PluginEngine::getUrl($this)));
        }
    }


    function getPluginName() {
        return _("Nutzer-Migrations-Tool (NuMiT)");
    }


    function show_action()
    {

        $GLOBALS['perm']->check("admin");

        Navigation::activateItem('/admin/tools/numit');

        if ($_REQUEST["cmd"] == "convert") {
            ob_start();
            $this->convert($_REQUEST["old_uname"], $_REQUEST["new_uname"], $_REQUEST["convert_ident"]);
            $output = ob_get_contents();
            ob_end_clean();
        }

        
        $factory = new Flexi_TemplateFactory(dirname(__FILE__)."/templates");
        $template = $factory->open("numit"); 
        $template->set_layout('layout.php');
        $template->set_attribute('output', $output);
        $template->set_attribute('plugin', $this);

        echo $template->render();

    }

    function remove_doubles($table, $field, $new_id, $old_id) {
        $db = new DB_Seminar();
        $db2 = new DB_Seminar();

        $db->query("SELECT a.$field FROM $table a, $table b WHERE a.user_id = '$new_id' AND b.user_id = '$old_id' AND a.$field = b.$field");
        while ($db->next_record()) {
            $db2->query("DELETE FROM $table WHERE $field = '".$db->f($field)."' AND user_id = '$new_id'");
        }
    }


    /*
     * Konvertiert einen Nutzer
     *	old_user: Name des Nutzers, der die Daten hat
     *	new_user: Name des Nutzers, der die Daten erhlt
     *	                Gstebucheintrge, Eigene Kategorien, etc. mit bertragen
     */
    function convert($old_user, $new_user) {
        $db = new DB_Seminar();
        $db2 = new DB_Seminar();

        // User-ID des zu konvertierenden Users auslesen
        $db->query("SELECT user_id FROM auth_user_md5 WHERE username = '$old_user'");
        if (!$db->next_record()) {
            echo MessageBox::error("Der Benutzername \"$old_user\" existiert nicht!");
            return FALSE;
        }

        $old_id = $db->f("user_id");

        // User-ID des Users, der upgedatet / gendert wird
        $db->query("SELECT user_id FROM auth_user_md5 WHERE username = '$new_user'");
        if (!$db->next_record()) {
            echo MessageBox::error("Der Benutzername \"$new_user\" existiert nicht!");
            return FALSE;
        }

        $new_id = $db->f("user_id");

        /*
         * Identittsrelevante Daten migrieren
         */
        //Namen bertragen
        $db->query("SELECT Vorname, Nachname FROM auth_user_md5 WHERE user_id = '$old_id'");
        $db->next_record();
        $db->query("UPDATE auth_user_md5 SET Vorname = '".$db->f("Vorname")."', Nachname = '".$db->f("Nachname")."' WHERE user_id = '$new_id'");

        //Aktive Session
        //$db->query("DELETE FROM active_sessions WHERE sid = '$new_id'");
        //$db->query("UPDATE active_sessions SET sid='$new_id' WHERE sid = '$old_id'");

        // Veranstaltungseintragungen
        $this->remove_doubles("seminar_user", "Seminar_id", $new_id, $old_id);
        $db->query("UPDATE seminar_user SET user_id = '$new_id' WHERE user_id = '$old_id';");

        $this->remove_doubles("admission_seminar_user", "seminar_id", $new_id, $old_id);
        $db->query("UPDATE admission_seminar_user SET user_id = '$new_id' WHERE user_id = '$old_id';");

        // Persnliche Infos
        $db->query("DELETE FROM user_info WHERE user_id = '$new_id';");
        $db->query("UPDATE user_info SET user_id = '$new_id' WHERE user_id = '$old_id';");

        // Studiengnge
        $this->remove_doubles("user_studiengang", "studiengang_id", $new_id, $old_id);
        $db->query("UPDATE user_studiengang SET user_id = '$new_id' WHERE user_id = '$old_id';");

        // Gstebuch
        $db->query("UPDATE guestbook SET user_id = '$new_id' WHERE user_id = '$old_id';");
        $db->query("UPDATE guestbook SET range_id = '$new_id' WHERE range_id = '$old_id';");

        // Eigene Kategorien
        $db->query("UPDATE kategorien SET range_id = '$new_id' WHERE range_id = '$old_id';");

        // Institute
        $this->remove_doubles("user_inst", "Institut_id", $new_id, $old_id);
        $db->query("UPDATE user_inst SET user_id = '$new_id' WHERE user_id = '$old_id';");

        //Generische Datenfelder
        $db->query("DELETE FROM datafields_entries WHERE range_id = '$new_id'");
        $db->query("UPDATE datafields_entries SET range_id = '$new_id' WHERE range_id = '$old_id'");

        //Buddys
        $db->query("UPDATE contact SET owner_id='$new_id' WHERE owner_id = '$old_id'");

        //WAP-Sessions
        $db->query("DELETE FROM wap_sessions WHERE user_id = '$new_id'");
        $db->query("UPDATE wap_sessions SET user_id='$new_id' WHERE user_id = '$old_id'");

        /*
         * Restliche Daten bertragen
         */
        // Forumsbeitrge
        $db->query("UPDATE px_topics SET user_id='$new_id' WHERE user_id = '$old_id'");

        // Dateieintragungen und Ordner
        $db->query("UPDATE dokumente SET user_id='$new_id' WHERE user_id = '$old_id'");
        $db->query("UPDATE folder SET user_id='$new_id' WHERE user_id = '$old_id'");

        //Kalender
        $db->query("UPDATE calendar_events SET range_id='$new_id' WHERE range_id = '$old_id'");
        $db->query("UPDATE calendar_events SET autor_id='$new_id' WHERE autor_id = '$old_id'");

        //Archiv
        $this->remove_doubles("archiv_user", "seminar_id", $new_id, $old_id);
        $db->query("UPDATE archiv_user SET user_id='$new_id' WHERE user_id = '$old_id'");

        //Evaluationen
        $db->query("UPDATE eval SET author_id='$new_id' WHERE author_id = '$old_id'");

        $this->remove_doubles("eval_user", "eval_id", $new_id, $old_id);
        $this->remove_doubles("evalanswer_user", "evalanswer_id", $new_id, $old_id);
        $db->query("UPDATE eval_user SET user_id='$new_id' WHERE user_id = '$old_id'");
        $db->query("UPDATE evalanswer_user SET user_id='$new_id' WHERE user_id = '$old_id'");

        //Literatur
        $db->query("UPDATE lit_catalog SET user_id='$new_id' WHERE user_id = '$old_id'");
        $db->query("UPDATE lit_list SET range_id='$new_id' WHERE range_id = '$old_id'");

        //Nachrichten (Interne)
        $db->query("UPDATE message SET autor_id='$new_id' WHERE autor_id = '$old_id'");

        $this->remove_doubles("message_user", "message_id", $new_id, $old_id);
        $db->query("UPDATE message_user SET user_id='$new_id' WHERE user_id = '$old_id'");

        //News
        $db->query("UPDATE news SET user_id='$new_id' WHERE user_id = '$old_id'");
        $db->query("UPDATE news_range SET range_id='$new_id' WHERE range_id = '$old_id'");

        //Referenztabelle: Abstimmungen, etc.
        $this->remove_doubles("object_user", "object_id", $new_id, $old_id);
        $db->query("UPDATE object_user SET user_id='$new_id' WHERE user_id = '$old_id'");

        //Simpel Content Module (Simple Wiki ;)
        $db->query("UPDATE scm SET user_id='$new_id' WHERE user_id = '$old_id'");

        //Statusgruppeneintrge	
        $this->remove_doubles("statusgruppe_user", "statusgruppe_id", $new_id, $old_id);
        $db->query("UPDATE statusgruppe_user SET user_id='$new_id' WHERE user_id = '$old_id'");

        //Termine
        $db->query("UPDATE termine SET autor_id='$new_id' WHERE autor_id = '$old_id'");

        //Votings
        $db->query("UPDATE vote SET author_id='$new_id' WHERE author_id = '$old_id'");
        $db->query("UPDATE vote SET range_id='$new_id' WHERE range_id = '$old_id'");

        $this->remove_doubles("vote_user", "vote_id", $new_id, $old_id);
        $db->query("UPDATE vote_user SET user_id='$new_id' WHERE user_id = '$old_id'");

        $this->remove_doubles("voteanswers_user", "answer_id", $new_id, $old_id);
        $db->query("UPDATE voteanswers_user SET user_id='$new_id' WHERE user_id = '$old_id'");

        //Wiki
        $db->query("UPDATE wiki SET user_id='$new_id' WHERE user_id = '$old_id'");
        $db->query("UPDATE wiki_locks SET user_id='$new_id' WHERE user_id = '$old_id'");

        //Adressbucheintrge
        $db->query("UPDATE contact SET owner_id = '$new_id' WHERE owner_id = '$old_id'");

        echo MessageBox::info("Datenkonvertierung wurde durchgef&uuml;hrt ($old_user -&gt; $new_user).");

        $messaging = new messaging;

        $message = sprintf(_("Sehr geehrter Nuter, sehr geehrte Nutzerin, \n\n die Daten wurden des Accounts %s  wurden auf den Account %s  bertragen. 
                    Nutzen Sie bitte zuknftig den neuen Account %s. \n\n Kursmanager"),$old_user,$new_user,$new_user);

        $subject = _("Nutzeraccount wurde migriert");

        $messaging->insert_message($message,$new_user,'','','','','',$subject,'');


        return true;
    }
}
